(***********************************************************************)
(*                                                                     *)
(*                              Ref-image                              *)
(*                                                                     *)
(*                  Image Segmentation by Refinement                   *)
(*                                                                     *)
(*               EPI Pomdapi, INRIA Paris - Rocquencourt               *)
(*                                                                     *)
(*  Copyright 2009-2013 INRIA.                                         *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Hend Ben Ameur <hbenameur@yahoo.ca>                                *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: display.ml,v 1.14 2013-12-02 15:28:38 fclement Exp $ *)

module Nat = Unatural;;

module Set = Umatrix_nonempty_set;;
module Mapping = Umatrix_graph;;
module Structural_mapping = Umatrix_structural_mapping;;
module Config = Configuration;;

let print_report ident printer ppf x =
  let b = Config.print_report (Config.print_ident_value ident printer) ppf x in
  Format.fprintf ppf "@.@.";
  b
;;

open Configuration_types;;
open Segmentation_spec;;
open Display_types;;
open Display_types_print;;
open Display_window;;

module Make =
  functor (Scal : SCALAR) ->
  functor (Cod : CODOMAIN) ->
  functor (St : STATE with
             type scalar = Scal.t and
             type 'a codomain = 'a Cod.t) ->
  functor (H : History_spec.HISTORY with
             type state = St.t) ->
struct
  type history = H.t;;
  
  module Dq = Display_quantity.Make (Scal) (St);;
  
  (* Get configuration. *)
  
  let ppf = Config.get_formatter ();;
  
  let sz = (Config.get_image_width (), Config.get_image_height ());;
  assert (print_report "image_size" print_image_size ppf sz);;
  
  let incremental_mode = Config.get_incremental_mode ();;
  
  let { on = graphics; kind = gk; } = Config.get_graphics_mode ();;
  let get_gk () =
    match gk with
    | None -> assert false
    | Some gk -> gk
  ;;
  
  let min_w_sz = Config.get_graphics_window_min_size ()
  and max_w_sz = Config.get_graphics_window_max_size ()
  ;;
  
  let thres = Config.get_graphics_gradient_threshold ();;
  
  let draw set_and_draw q zsh =
    let St.State zs = H.head_state zsh in
    let zt = Cod.to_triple zs in
    set_and_draw q zt;
    if incremental_mode then Monitor.interruption ppf;
  ;;
  
  let init_quantity = draw Dq.set_quantity_and_draw;;
  
  let init zsh =
    if graphics then
    begin
      Dq.setup thres (get_gk ()) min_w_sz max_w_sz sz;
      let { window_size = w_sz; _; } = get_graphics_elements () in
      assert (print_report "window_size" print_window_size ppf w_sz);
      init_quantity Data zsh;
      if H.head_state_number zsh > Nat._0_ then
      begin (* This means warm restart just occurred. *)
        init_quantity Param zsh;
        init_quantity Grad zsh;
      end;
    end;
  ;;
  
  let display_quantity = draw Dq.update_quantity_and_draw;;
  
  let display zsh =
    if graphics then
    begin
      display_quantity Param zsh;
      display_quantity Grad zsh;
    end;
  ;;
end
;;
